// #autoload
// #name = HudManager
// #version = 1.1.0
// #date = October 25, 2003
// #description = Allows you to reposition, resize, hide/show, and dock on-screen HUDs
// #category = ilys' Scripts
// #credits = Labrat, whose HudMover was the basis of and inspiration to this script.
// #credits = Uberguy, for the original HudManager
// #readme = scripts/UberGuy/HudManager.txt
// #include = autoload 2.2.1
// #include = support/map.cs
// #include = support/vector.cs
// #include = support/callback.cs
// #include = support/events.cs
// #include = support/docking_tools.cs
// #include = support/mission_callbacks.cs
// #include = support/bind_manager.cs 1.1.0

//=============================================================================
//	Setup operations
//=============================================================================

if(!isObject(HudManager))
{
	new ScriptObject("HudManager")
	{
		class = "HudManager";
		displayed = false;
		hudMap = Container::newVectorMap();
		hudReverseMap = Container::newVectorMap();
		hudNoResizeList = Container::newVectorMap();
		layoutVersion = "3.2";
		SystemHudsInitialized = "0";
	};
}

$HudManager::tempObject = new FileObject();
$HudManager::tempObject.openForWrite("prefs/HudManager/");
$HudManager::tempObject.close();
$HudManager::tempObject.delete();
$HudManager::tempObject = "";

exec("prefs/HudManager/HudManagerPrefs.cs");
if($HudManager::currentCfg $= "") $HudManager::currentCfg = "default";
if($HudManager::comeBack $= "") $HudManager::comeBack = true;
if($HudManager::useGhost $= "") $HudManager::useGhost = true;

//=============================================================================
//	HudManager Methods and supporting calls
//=============================================================================

function HudManagerDataBloc::read(%this)
{
	if((%this.control $= "") || !isObject(%this.control)) return;
	%this.extent = %this.control.extent;
	%this.position = %this.control.position;
}

function HudManagerDataBloc::write(%this)
{
	if((%this.control $= "") || !isObject(%this.control)) return;
	if(%this.position !$= "") %this.control.setPosition2(%this.position);
	if((!%this.noResize && (%this.extent !$= ""))) %this.control.setExtent2(%this.extent);
}

function HudManager::forceToggleGui(%this, %val)
{
	%this.displayed = %val;
	if(HudManagerBackGui.isMember(HudManagerDockGui)) %this.cancelDock();
	%this.updateGUI();
}

function HudManager::toggleGui(%this)
{
	%this.displayed = (!%this.displayed);
	if(HudManagerBackGui.isMember(HudManagerDockGui)) %this.cancelDock();
	%this.updateGUI();
}

function HudManager::populateGUI(%this)
{
	HM_controlList.clear();
	for(%i = 0; %i < %this.hudMap.size(); %i++)
	{
		%key = %this.hudMap.keys.valueAt(%i);
		%bloc = %this.hudMap.value(%key);
		HM_controlList.addRow(%i,%bloc.name);
		if(isObject(%bloc.control))
		{
			if(%bloc.canvas $= "" && %bloc.control.getGroup() != -1) %bloc.canvas = %bloc.control.getGroup().getName();
			if(%bloc.control.getGroup() == -1) %bloc.show = 0;
			else %bloc.show = 1;
		}
		%style = 1;
		if(%bloc.noResize) %style = 3;
		if(!%bloc.canvas.isMember(%bloc.control)) %style = 4;
		if(!isObject(%bloc.control)) %style = 2;
		HM_controlList.setRowStyle(%i, %style);
	}
	HM_controlList.sort(0,true);
	if(%this.lastID !$= "") HM_controlList.setSelectedById(%this.lastID);
}

function HudManager::toggleSelectedResize(%this)
{
	%id = HM_controlList.getSelectedId();
	%bloc = %this.getHudByName(HM_controlList.getRowTextById(%id));
	if(isObject(%bloc) && isObject(%bloc.control))
	{
		if(!%bloc.show) return;
		%bloc.noResize = !%bloc.noResize;
		if(%bloc.canvas $= "" && isObject(%bloc.control) && %bloc.control.getGroup() > 0) %bloc.canvas = %bloc.control.getGroup().getName();
		%style = 1;
		if(%bloc.noResize) %style = 3;
		if(!%bloc.canvas.isMember(%bloc.control)) %style = 4;
		if(!isObject(%bloc.control)) %style = 2;
		HM_controlList.setRowStyle(%id, %style);
		%this.lastID = %id;
		%this.toggleGui();
		%this.toggleGui();
		HM_ResizeToggleButton.makeFirstResponder(true);
	}
}

function HudManager::toggleShowHud(%this)
{
	%id = HM_controlList.getSelectedId();
	%bloc = %this.getHudByName(HM_controlList.getRowTextById(%id));
	if(isObject(%bloc) && isObject(%bloc.control))
	{
		%bloc.show = !%bloc.show;
		if(%bloc.canvas $= "" && isObject(%bloc.control) && %bloc.control.getGroup() > 0) %bloc.canvas = %bloc.control.getGroup().getName();
		if(%bloc.show) %bloc.canvas.add(%bloc.control);
		else %bloc.canvas.remove(%bloc.control);
		%isShow = %bloc.show ? "Hide HUD" : "Show HUD";
		HM_ShowHudToggleButton.setValue(%isShow);
		%style = 1;
		if(%bloc.noResize) %style = 3;
		if(!%bloc.canvas.isMember(%bloc.control)) %style = 4;
		if(!isObject(%bloc.control)) %style = 2;
		HM_controlList.setRowStyle(%id, %style);
		%this.lastID = %id;
		%this.toggleGui();
		%this.toggleGui();
		HM_ShowHudToggleButton.makeFirstResponder(true);
	}
}

function HudManager::updateGUI(%this)
{
	if(%this.displayed)
	{
		HudManagerGui.setVisible(true);
		Canvas.pushDialog(HudManagerGui);
		if(!%this.guiInitialized)
		{
	      		HM_controlList.setSortColumn(0);
	      		HM_controlList.setSortIncreasing(true);
	      		%w = firstWord(HM_controlList.extent);
	      		HM_controlList.addColumn(0,"HUD Name",%w,%w,%w);
			HM_controlList.addStyle(1,$ShellFont,$ShellFontSize,"80 220 200", "30 255 225", "10 60 40");
			HM_controlList.addStyle(2,$ShellFont,$ShellFontSize,"120 120 120", "120 120 120", "120 120 120");
			HM_controlList.addStyle(3,$ShellFont,$ShellFontSize,"250 120 120", "250 120 120", "250 120 120");
			HM_controlList.addStyle(4,$ShellFont,$ShellFontSize,"120 120 250", "120 120 250", "60 60 60");
			%this.guiInitialized = true;
		}
		%this.populateGUI();
		HM_MoveButton.makeFirstResponder(true);
	}
	else Canvas.popDialog(HudManagerGui);
}

function HudManager::removeListedHud(%this)
{
	%hud = HM_controlList.getRowTextById(HM_controlList.getSelectedId());
	%this.removeHUD(%hud);
	%this.lastID = "";
	%this.populateGUI();
}

function HudManager::displayDockingGui(%this, %dataBloc)
{
	%this.dockDisplayed = true;
	%control = %dataBloc.control;
	if(!isObject(HudManagerDockGui)) exec("scripts/UberGuy/HudManagerDock.gui");
	HudManagerBackGui.add(HudManagerDockGui);
	HudManagerBackGui.setText("Docking Target List");
	if(!%this.dockInitialized)
	{
  		HM_VdockList.setSortColumn(0);
  		HM_HdockList.setSortColumn(0);
  		HM_VdockList.setSortIncreasing(true);
  		HM_HdockList.setSortIncreasing(true);
		%w = firstWord(HM_VdockList.extent);
  		HM_VdockList.addColumn(0,"Vertical Target HUD Name",%w,%w,%w);
  		HM_HdockList.addColumn(0,"Horizontal Target HUD Name",%w,%w,%w);
		HM_VdockList.addStyle(1,$ShellFont,$ShellFontSize,"80 220 200", "30 255 225", "10 60 40");
		HM_HdockList.addStyle(1,$ShellFont,$ShellFontSize,"80 220 200", "30 255 225", "10 60 40");
		%this.dockInitialized = true;
	}
	HM_VdockList.clear();
	HM_VdockList.addRow(0,"*NOTHING*");
	HM_VdockList.setRowStyle(0, 1);
	for(%i = 1; %i <= %this.hudMap.size(); %i++)
	{
		%key = %this.hudMap.keys.valueAt(%i-1);
		%bloc = %this.hudMap.value(%key);
		if(DockManager.isValidDockTarget(%control,%bloc.control))
		{
			HM_VdockList.addRow(%i,%bloc.name);
			HM_VdockList.setRowStyle(%i, 1);
			%info = DockManager.getDockInfo(%control,"V");
			if(isObject(%info))
			{
				%tgt = %info.target;
				if(isObject(%tgt) && isObject(%bloc.control) && (%tgt.getId() == %bloc.control.getID())) HM_VdockList.setSelectedById(%i);
			}
			else HM_VdockList.setSelectedById(0);
		}
	}

	HM_VdockList.onSelect();
	HM_VdockList.sort(0);
	HM_VdockList.sort(0);
	HM_HdockList.clear();
	HM_HdockList.addRow(0,"*NOTHING*");
	HM_HdockList.setRowStyle(0, 1);
	for(%i = 1; %i <= %this.hudMap.size(); %i++)
	{
		%key = %this.hudMap.keys.valueAt(%i-1);
		%bloc = %this.hudMap.value(%key);
		if(DockManager.isValidDockTarget(%control, %bloc.control))
		{
			HM_HdockList.addRow(%i,%bloc.name);
			HM_HdockList.setRowStyle(%i, 1);
			%info = DockManager.getDockInfo(%control,"H");
			if(isObject(%info))
			{
				%tgt = %info.target;
				if(isObject(%tgt) && isObject(%bloc.control) && (%tgt.getId() == %bloc.control.getID())) HM_HdockList.setSelectedById(%i);
			}
			else HM_HdockList.setSelectedById(0);
		}
	}
	HM_HdockList.onSelect();
	HM_HdockList.sort(0);
	HM_HdockList.sort(0);
	HudManagerDockGui.currentControl = %control;
}

function HM_controlList::onSelect(%this, %id, %text)
{
	%bloc = HudManager.getHudByName(HM_controlList.getRowTextById(%id));
	%isShow = %bloc.show ? "Hide HUD" : "Show HUD";
	HM_ShowHudToggleButton.setValue(%isShow);
	HudManager.lastID = %id;
	if($HudManager::useGhost)
	{
		if(!isObject(SelectionGhost))
		{
			new HudBitmapCtrl(SelectionGhost)
			{
				profile = "GuiContentProfile";
				position = "0 0 ";
				extent = "1 1 ";
				minExtent = "0 0 ";
				visible = "0";
				hideCursor = "0";
				bypassHideCursor = "0";
				helpTag = "0";
				fillcolor = "1 1 1";
				framecolor = "0 0 0";
				opacity = 0.25;
			};
		}
		if(isObject(%bloc.canvas)) %bloc.canvas.add(SelectionGhost);
		else playgui.add(SelectionGhost);
		if(%bloc.extent $= "") %bloc.extent = %bloc.name.extent;
		if(%bloc.position $= "") %bloc.position = %bloc.name.position;
		SelectionGhost.extent = %bloc.extent;
		SelectionGhost.position = %bloc.position;
		SelectionGhost.setVisible(true);
	}
	else
	{
		if(isObject(SelectionGhost)) SelectionGhost.setVisible(false);
	}
}

function HM_VdockList::onSelect(%this, %id, %text)
{
	%id = HM_controlList.getSelectedId();
	%obj = HudManager.getHudByName(HM_controlList.getRowTextById(%id)).control;
	%id = HM_VdockList.getSelectedId();
	%selected = HudManager.getHudByName(HM_VdockList.getRowTextById(%id)).control;
	%info = DockManager.getDockInfo(%obj,"V");
	if(isObject(%info)) %target = %info.target;
	if(isObject(%selected) && isObject(%target) && (%target.getID() == %selected.getID()))
	{
		HM_VT.setValue(%info.tgtSide $= "T");
		HM_VB.setValue(%info.tgtSide $= "B");
		HM_CVT.setValue(%info.ctrlSide $= "T");
		HM_CVB.setValue(%info.ctrlSide $= "B");
	}
	else
	{
		HM_VT.setValue(false);
		HM_VB.setValue(false);
		HM_CVT.setValue(false);
		HM_CVB.setValue(false);
	}
}

function HM_HdockList::onSelect(%this, %id, %text)
{
	%id = HM_controlList.getSelectedId();
	%obj = HudManager.getHudByName(HM_controlList.getRowTextById(%id)).control;
	%id = HM_HdockList.getSelectedId();
	%selected = HudManager.getHudByName(HM_HdockList.getRowTextById(%id)).control;
	%info = DockManager.getDockInfo(%obj,"H");
	if(isObject(%info)) %target = %info.target;
	if(isObject(%selected) && isObject(%target) && (%target.getID() == %selected.getID()))
	{
		HM_HL.setValue(%info.tgtSide $= "L");
		HM_HR.setValue(%info.tgtSide $= "R");
		HM_CHL.setValue(%info.ctrlSide $= "L");
		HM_CHR.setValue(%info.ctrlSide $= "R");
	}
	else
	{
		HM_HL.setValue(false);
		HM_HR.setValue(false);
		HM_CHL.setValue(false);
		HM_CHR.setValue(false);
	}
}

function HudManager::dockButton(%this)
{
	%id = HM_controlList.getSelectedId();
	%obj = %this.getHudByName(HM_controlList.getRowTextById(%id));
	%this.lastID = %id;
	if(!isObject(%obj) || !isObject(%obj.control)) return;
	%this.displayDockingGui(%obj);
}

function HudManager::dock(%this)
{
	%v = (HM_VT.getValue() ? "T" : "B");
	%cv = (HM_CVT.getValue() ? "T" : "B");
	%h = (HM_HL.getValue() ? "L" : "R");
	%ch = (HM_CHL.getValue() ? "L" : "R");
	%id = HM_controlList.getSelectedId();
	%obj = %this.getHudByName(HM_controlList.getRowTextById(%id)).control;
	%id = HM_VdockList.getSelectedId();
	%nm = HM_VdockList.getRowTextById(%id);
	if(%nm $= "*NOTHING*") DockManager.unDock(%obj,"V");
	else
	{
		%target = %this.getHudByName(%nm).control;
		%offset = (HM_CVT.getValue() ? %obj.getTop() : %obj.getBottom()) -
			(HM_VT.getValue() ? %target.getTop() : %target.getBottom());
		DockManager.dock(%obj,%cv,%target,%v,%offset);
	}
	%id = HM_HdockList.getSelectedId();
	%nm = HM_HdockList.getRowTextById(%id);
	if(%nm $= "*NOTHING*") DockManager.unDock(%obj,"H");
	else
	{
		%target = %this.getHudByName(HM_HdockList.getRowTextById(%id)).control;
		%offset = (HM_CHL.getValue() ? %obj.getLeft() : %obj.getRight()) -
			(HM_HL.getValue() ? %target.getLeft() : %target.getRight());
		DockManager.dock(%obj,%ch,%target,%h, %offset);
	}
	HudManagerBackGui.remove(HudManagerDockGui);
	HudManagerBackGui.setText("HudManager");
	%this.dockDisplayed = false;
	HM_DockButton.makeFirstResponder(true);
}

function  HudManager::cancelDock(%this)
{
	HudManagerBackGui.remove(HudManagerDockGui);
	HudManagerBackGui.setText("HudManager");
	%this.dockDisplayed = false;
	HM_DockButton.makeFirstResponder(true);
}

function HudManager::updateDefinitions(%this)
{
	for(%i = 0; %i < %this.hudMap.size(); %i++)
	{
		%key = %this.hudMap.keys.valueAt(%i);
		%this.hudMap.value(%key).read();
	}
}

function HudManager::createGhost(%this, %original)
{
	if(!isObject(%this.ghost))
	{
		%this.ghost = new HudBitmapCtrl()
		{
			profile = "GuiContentProfile";
			position = "0 0 ";
			extent = "1 1 ";
			minExtent = "0 0 ";
			visible = "0";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";
			bitmap = "gui/dlg_fieldfill.png";
			opacity = 0.7;
		};
	}
	%this.ghost.extent = %original.extent;
	%this.ghost.position = %original.position;
}

function HudManager::setHuds(%this)
{
	if(Canvas.getContent().getID() != PlayGui.getID()) return;
	for(%i = 0; %i < %this.hudMap.size(); %i++)
	{
		%key = %this.hudMap.keys.valueAt(%i);
		%bloc = %this.hudMap.value(%key);
		%bloc.write();
		%info = DockManager.getDockInfo(%bloc.control,"H");
		if(isObject(%info)) DockManager.align(%bloc.control,%info.ctrlSide,%info.target,%info.tgtSide,%info.offset);
		%info = DockManager.getDockInfo(%bloc.control,"V");
		if(isObject(%info)) DockManager.align(%bloc.control,%info.ctrlSide,%info.target,%info.tgtSide,%info.offset);
	}
	if(isObject(uberChat)) uberChat.setWidth();
}

function HudManager::clear(%this)
{
	while (%this.hudMap.size())
	{
		%key = %this.hudMap.keys.valueAt(0);
		%this.hudMap.remove(%key);
	}
}

function HudManager::addHUD(%this, %name, %control, %noResize, %position, %extent, %show, %canvas)
{
	if(%control $= "")
	{
		error("HudManager::addHUD -> unable to add unnamed GUI objects to Hud Manager.");
		return false;
	}
	if(%show $= "") %show = 1;
	if(%canvas $= "" && isObject(%control) && %control.getGroup() > 0) %canvas = %control.getGroup().getName();
//	else %canvas = "playgui";
	if(%position $= "") %position = %control.position;
	if(%extent $= "") %extent = %control.extent;
	if(!%this.hudMap.hasKey(%control))
	{
		warn("HudManager::addHUD -> added Hud \"" @ %control @ "\" as \"" @ %name @ "\".");
		%db = new ScriptObject() {
			class = "HudManagerDataBloc";
			name     = %name;
			control  = %control;
			extent   = %extent;
			position = %position;
			noResize = %noResize;
			show     = %show;
			canvas   = %canvas;
		};
	}
	else
	{
		warn("HudManager::addHUD -> replacing existing managed Hud \"" @ %name @ "\".");
		%db = %this.getHud(%control);
		if(%this.name     !$= "") %db.name     = %name;
		if(%this.control  !$= "") %db.control  = %control;
		if(%this.extent   !$= "") %db.extent   = %extent;
		if(%this.position !$= "") %db.position = %position;
		if(%this.noResize !$= "") %db.noResize = %noResize;
		if(%this.Show     !$= "") %db.Show     = %Show;
		if(%this.Canvas   !$= "") %db.Canvas   = %Canvas;
	}
	%this.hudMap.add(%control, %db);
	%this.hudReverseMap.add(%name, %control);
	if(%canvas !$= "" && isObject(%control))
	{
		if(%show == 0 && %canvas.isMember(%control)) %canvas.remove(%control);
		else if(%show == 1 && !%canvas.isMember(%control) && %control !$= "hudClusterBack" && %control !$= "ClockHud") %canvas.add(%control);
	}
	return true;
}

function HudManager::initSystemHuds(%this)
{
	HudManager.addHUD("ChatHUD", OuterChatHud.getGroup().getName(), true,"","","","MainChatHud");
	HudManager.addHUD("InventoryHUD", "inventoryHud","","","","","PlayGui");
	HudManager.addHUD("ObjectiveHUD", "objectiveHud","","","","","PlayGui");
	HudManager.addHUD("BackPackHUD", "backpackFrame","","","","","PlayGui");
	HudManager.addHUD("WeaponHUD", "weaponsHud", true,"","","","PlayGui");
	HudManager.addHUD("ClockHUD", "clockHud","","","","","PlayGui");
	HudManager.addHUD("Compass Cluster", "hudClusterBack","","","","","PlayGui");
	HudManager.addHUD("VoiceCommHUD", "voiceCommHud","","","","","PlayGui");
	HudManager.addHUD("v-Chat Menu", "chatMenuHud", true,"","","","ChatMenuHudDlg");
	HudManager.addHUD("Net Graph HUD", "NetGraphHudFrame","","","","","PlayGui");
	HudManager.addHUD("Ping Bar HUD", "NetBarHudFrame","","","","","PlayGui");
	HudManager.addHUD("Center Print Bar", "CenterPrintDlg","","","","","PlayGui");
	HudManager.addHUD("Bottom Print Bar", "bottomPrintDlg","","","","","PlayGui");
	HudManager.addHUD("Vehicle Dashboard", "dashboardHud","","","","","PlayGui");
	HudManager.addHUD("AmmoHUD", "ammoHud","","","","","retCenterHud");
	HudManager.addHUD("TR2 Jackpot HUD", "TR2BonusHud", true,"","","","PlayGui");
	HudManager.addHUD("TR2 Score HUD", "TR2EventPopup", true,"","","","PlayGui");
	HudManager.addHUD("Message Hud", "messageHud_Frame","","","","","MessageHud");
	HudManager.addHUD("Vote Bar", "mainVoteHud","","","","","_OuterChat");
	%this.SystemHudsInitialized = 1;
	%this.schedule(500,"setHuds");
}

function HudManager::removeHUD(%this, %name, %noDelete)
{
	%control = %this.hudReverseMap.value(%name);
	if((%control $= "") || !%this.hudMap.hasKey(%control))
	{
		warn("HudManager::removeHUD -> attempt to remove \"" @ %name @ "\" from Hud Manager - not a managed name");
		return false;
	}
	%temp = %this.hudMap.value(%control);
	%this.hudMap.remove(%control);
	%this.hudReverseMap.remove(%name);
	if(!%noDelete) %temp.delete();
	return true;
}

function HudManager::getHudByName(%this, %name)
{
	%tmp =  %this.hudMap.value(%this.hudReverseMap.value(%name));
	if(isObject(%tmp)) return %tmp;
	else return "";
}

function HudManager::getHud(%this, %control)
{
	%tmp = %this.hudMap.value(%control);
	if(isObject(%tmp)) return %tmp;
	else return "";
}

function HudManager::moveHudStub(%this)
{
	%id  = HM_controlList.getSelectedId();
	%obj = %this.getHudByName(HM_controlList.getRowTextById(%id));
	if(!isObject(%obj) || !isObject(%obj.control)) return;
	%this.lastID = %id;
	%this.forceToggleGui(false);
	%this.moveHud(%obj);
}

function HudManager::moveHud(%this, %dataBloc)
{
	if(!isObject(%dataBloc))
	{
		error("HudManager::moveHud -> null object reference passed");
		return;
	}

	if(%this.isOnPlayGui(%databloc.control)) %parent = %dataBloc.control.getGroup();
	else %parent = playGui;
	if($HudManager::useGhost || (%dataBloc.control.visible == false) || !%this.isOnPlayGui(%databloc.control))
	{
		%this.createGhost(%dataBloc.control);
		%this.currentHud = %this.ghost;
		%this.ghost.setVisible(true);
		if(!%parent.isMember(%this.ghost)) %parent.add(%this.ghost);
	}
	else %this.currentHud = %dataBloc.control;
	%this.currentDataBloc = %dataBloc;
	HM_moveHud(1);
}

function HudManager::readMapFile(%this, %mapname)
{
	%reader = new FileObject();
	%filename = "prefs/HudManager/" @ %mapname @ ".layout";
	if(!isFile(%filename))
	{
		error("HudManager::readMapFile -> Could not load file \"" @ %filename @ "\".");
		return false;
	}
	%reader.openForRead(%filename);
	%firstLine = %reader.readLine();
	%version = trim(getWord(%firstLine,getWordCount(%firstLine)-1));
	if((trim(firstWord(%firstLine)) !$= "Layout") || (%version !$= %this.layoutVersion)) MessageBoxOK("HudManager Error", "You have loaded a file in an old version of the HudMangager save file format, which may not be compatible with the current version of this script","");
	while (!%reader.isEOF())
	{
		%line = %reader.readLine();
		if(getSubStr(%line,0,1) !$= ";")
		{
			if(getFieldCount(%line) > 12)
			{
				%control = getField(%line,1);
				%this.addHUD(getField(%line,0),%control,getField(%line,2),
					getField(%line,3),getField(%line,4),getField(%line,13),getField(%line,14));

				if(getField(%line, 6) !$= " ") DockManager.dock(%control,getField(%line,5),getField(%line, 6),getField(%line, 7),getField(%line, 8));
				if(getField(%line,10) !$= " ") DockManager.dock(%control,getField(%line,9),getField(%line,10),getField(%line,11),getField(%line,12));
			}
		}
	}
	%reader.close();
	%reader.delete();
	$HudManager::currentCfg = %mapname;
}

function HudManager::writeMapFile(%this, %mapname)
{
	%writer = new FileObject();
	%filename = "prefs/HudManager/" @ %mapname @ ".layout";
	%writer.openForWrite(%filename);
	%writer.writeline("Layout format version:" SPC %this.layoutVersion);
	for(%i = 0; %i < %this.hudMap.size(); %i++)
	{
		%key = %this.hudMap.keys.valueAt(%i);
		%node = %this.hudMap.value(%key);
		%ctrl = %node.control;
		%H_info = DockManager.getDockInfo(%ctrl,"H");
		%V_info = DockManager.getDockInfo(%ctrl,"V");
		%H = isObject(%H_info);
		%V = isObject(%V_info);
		if(!%node.noResize) %node.noResize = false;
		if(%node.show $= "") %node.show = true;
		%writer.writeLine(
			%node.name			TAB 	%key				TAB
			%node.noResize			TAB	%node.position			TAB
			%node.extent			TAB	(%H ? %H_info.ctrlSide : "")	TAB
			(%H ? %H_info.target : " ")	TAB	(%H ? %H_info.tgtSide : " ")	TAB
			(%H ? %H_info.offset : " ")	TAB	(%V ? %V_info.ctrlSide : " ")	TAB
			(%V ? %V_info.target : " ")	TAB	(%V ? %V_info.tgtSide : " ")	TAB
			(%V ? %V_info.offset : " ")	TAB 	%node.show			TAB
			%node.canvas
		);
	}
	%writer.close();
	%writer.delete();
	$HudManager::currentCfg = %mapname;
}

function HudManager::load(%this, %file)
{
	%this.clear();
	%this.readMapFile(%file);
	%this.setHuds();
	if(%this.displayed)
	{
		%this.toggleGui();
		%this.toggleGui();
	}
}

function HudManager::LoadDialog(%this)
{
	ShellGetLoadFilename( "Load Hud Layout", "prefs/HudManager/*.layout", "", "HudManager.load", "" );
}

function HudManager::save(%this, %file)
{
	%this.writeMapFile(%file);
}

function HudManager::saveDialog(%this)
{
	ShellGetSaveFilename( "Save Hud Layout", "prefs/HudManager/*.layout", "", "HudManager.save", "" );
}

function HudManager::moverKeyMap(%this)
{
	if(!isObject(%this.bindMap))
	{
		%bind = moveMap.getBinding(yaw);
		%this.bindMap = new ActionMap(HM_MOVEMAP);
		%this.bindMap.bind(mouse, "xaxis", "S", 0.5, "HM_hudX");
		%this.bindMap.bind(mouse, "yaxis", "S", 0.5, "HM_hudY");
		%this.bindMap.bindCmd(mouse, button0, "", "HM_moveHud(1);" );
		%this.bindMap.bind(keyboard, "escape", HM_Reset);
		%this.bindMap.bind(keyboard, "tab" , HM_hudActionToggle);
		%this.bindMap.bind(keyboard, "lshift", HM_clipToggle);
		%this.bindMap.bindCmd(keyboard, "up", "HM_hudY_key(-1);", "HudManager.loopMove=0;");
		%this.bindMap.bindCmd(keyboard, "down", "HM_hudY_key(1);", "HudManager.loopMove=0;");
		%this.bindMap.bindCmd(keyboard, "left", "HM_hudX_key(-1);", "HudManager.loopMove=0;");
		%this.bindMap.bindCmd(keyboard, "right", "HM_hudX_key(1);", "HudManager.loopMove=0;");
	}
	if(isObject(moveMap)) moveMap.pop();
	if(isObject(passengerKeys)) passengerKeys.pop();
	if(isObject(observerBlockMap)) observerBlockMap.pop();
	if(isObject(observerMap)) observerMap.pop();
	if(isObject(ControlActionMap)) ControlActionMap.pop();
   	globalactionmap.pop();
	%this.bindMap.push();
}

function HudManager::checkHuds(%this)
{
	if(playGui.extent $= getWords(getResolution(),0,1))
	{
		%this.setHuds();
		return true;
	}
	else
	{
		%this.schedule(100, checkHuds);
		return false;
	}
}

function HudManager::isOnPlayGui(%this, %control)
{
	if(!isObject(%control)) return false;
	%pgID = playGui.getID();
	while (true)
	{
		%parent = %control.getGroup();
		if(isObject(%parent))
		{
			if(%parent.getID() == %pgID) return true;
			else %control = %parent;
		}
		else return false;
	}
}

function HudManager::checkIniFile(%this)
{
	%scriptPath = "scripts/UberGuy/HudManager.cs";
	%backupPath = $AutoloadIni @ ".bak";
	%fileObj = new FileObject();
	%lineNum = 0;
	%fileObj.openForRead($AutoloadIni);
	%foundFirstScript = false;
	while (!%fileObj.isEOF())
	{
		%l = trim(%fileObj.readLine());
		if((%l !$= "") && (getSubStr(%l,0,1) !$= ";"))
		{
			if(!%foundFirstScript)
			{
				if(%l $= %scriptPath)
				{
					%fileObj.close();
					%fileObj.delete();
					return false;
				}
				else
				{
					%line[%lineNum] = %scriptPath;
					%lineNum++;
				}
				%foundFirstScript = true;
			}
			else
			{
				if(%l $= %scriptPath) continue;
			}
		}
		%line[%lineNum] = %l;
		%lineNum++;
	}
	%fileObj.close();
	if(isFile(%backupPath)) deleteFile(%backupPath);
	renameFile($AutoloadIni,%backupPath);
	%fileObj = new FileObject();
	%fileObj.openForWrite($AutoloadIni);
	for (%i=0; %i < %lineNum; %i++) %fileObj.writeLine(%line[%i]);
	%fileObj.close();
	%fileObj.delete();
	return true;
}

function HudManager::changesDialog(%this, %force)
{
	if(!%force && %this.checkIniFile()) %this.schedule(1000,changesDialog,true);
	if(%force)
	{
		MessageBoxOKCancel("Changes Made", "<just:center><Font:Univers Condensed:16>" @
		"<color:2cacb5>HudManager changed the <color:ffffff>Autoload.ini " @
		"<color:2cacb5>file to run HudManager first. " @
		"Press OK to exit T2, then restart T2 to use the new setting. Press " @
		"CANCEL to continue. If you continue some HudMover-compatible HUDs may " @
		"not be listed until you restart. For problem HUDs that refuse to appear " @
		"in the Manager, see the in-game README on how to add them from the GUI.",
		"quit();","");
	}
}

function HudManagerGui::onWake(%this)
{
	if($HudManager::useGhost && isObject(SelectionGhost)) SelectionGhost.setVisible(true);
	else if(!$HudManager::useGhost && isObject(SelectionGhost)) SelectionGhost.setVisible(false);
}
function HudManagerGui::onSleep(%this)
{
	if(isObject(SelectionGhost)) SelectionGhost.setVisible(false);
}

function HM_refresh(%val)
{
	if(%val) HudManager.setHuds();
}

function HM_guiToggle(%val)
{
	if(%val) HudManager.toggleGui();
}

function HM_hudActionToggle(%val)
{
	if(%val)
	{
		if(HudManager.hudAction $= "move") HudManager.hudAction = "resize";
		else if(HudManager.hudAction $= "resize") HudManager.hudAction = "move";
	}
}

function HM_reset(%val)
{
	if(%val)
	{
		HudManager.hudAction = "";
		if(isObject(HudManager.bindMap)) HudManager.bindMap.pop();
		globalActionMap.push();
		if(Canvas.getContent() == PlayGui.getId()) updateActionMaps();
		if(isObject(HudManager.ghost))
		{
			if(HudManager.currentHud.getID() == HudManager.ghost.getID()) HudManager.ghost.setVisible(false);
		}
		HudManager.currentHud.setPosition2(HudManager.currentDataBloc.position);
		if(!HudManager.currentDataBloc.noResize && (HudManager.currentDataBloc.extent !$= "")) HudManager.currentHud.setExtent2(HudManager.currentDataBloc.extent);
		if($HudManager::comeBack) HudManager.forceToggleGui(true);
	}
}

function HM_clipToggle(%val)
{
	HudManager.constrain = !%val;
}

function HM_moveHud(%val)
{
	if(%val)
	{
		if(HudManager.hudAction !$= "")
		{
			HudManager.hudAction = "";
			if(isObject(HudManager.bindMap)) HudManager.bindMap.pop();
			globAlactionMap.push();
			if(Canvas.getContent() == PlayGui.getId()) updateActionMaps();
			if(isObject(HudManager.ghost))
			{
				if(HudManager.currentHud.getID() == HudManager.ghost.getID())
				{
					HudManager.currentDataBloc.extent = HudManager.ghost.extent;
					HudManager.currentDataBloc.position = HudManager.ghost.position;
					HudManager.currentDataBloc.write();
					HudManager.ghost.setVisible(false);
				}
			}
			if(!isObject(HudManager.ghost) || (HudManager.currentHud.getID() != HudManager.ghost.getID()))
			{
				HudManager.currentDataBloc.read();
				HudManager.currentHud = HudManager.currentDataBloc = "";
			}
			if($HudManager::comeBack) HudManager.forceToggleGui(true);
		}
		else
		{
			HudManager.hudAction = "move";
			HudManager.constrain = true;
			HudManager.moverKeyMap();
		}
	}
}

function HM_hudX(%val, %mode)
{
	if(isEventPending(HudManager.loopID)) cancel(HudManager.loopID);
	if((%mode == 2) && (!HudManager.loopMove)) return;
	if(HudManager.isOnPlayGui(HudManager.currentHud)) %parent = HudManager.currentHud.getGroup();
	else %parent = playGui;
	if(HudManager.hudAction $= "move")
	{
		%mx = getWord(%parent.extent,0) - getWord(HudManager.currentHud.extent,0);
		%x = getWord(HudManager.currentHud.position,0) + %val;
		if(HudManager.constrain) %x = (%x >= %mx) ? %mx : ((%x < 0) ? 0 : %x);
		HudManager.currentHud.setPosition(%x,getWord(HudManager.currentHud.position,1));
	}
	else if(HudManager.hudAction $= "resize")
	{
		if(HudManager.currentDataBloc.noResize) return;
		%mx = getWord(%parent.extent,0) - getWord(HudManager.currentHud.position,0);
		%x = getWord(HudManager.currentHud.extent,0) + %val;
		if(HudManager.constrain) %x = (%x >= %mx) ? %mx : ((%x < 0) ? 0 : %x);
		HudManager.currentHud.setExtent(%x,getWord(HudManager.currentHud.extent,1));
	}
	if((%mode != 0) && HudManager.loopMove) HudManager.loopID = schedule(200, 0, HM_hudX, %val, 2);
}

function HM_hudX_key(%val)
{
	HudManager.loopMove = true;
	HM_hudX(%val, 1);
}

function HM_hudY(%val, %mode)
{
	if(isEventPending(HudManager.loopID)) cancel(HudManager.loopID);
	if((%mode == 2) && (!HudManager.loopMove)) return;
	if(HudManager.isOnPlayGui(HudManager.currentHud)) %parent = HudManager.currentHud.getGroup();
	else %parent = playGui;
	if(HudManager.hudAction $= "move")
	{
		%my = getWord(%parent.extent,1) - getWord(HudManager.currentHud.extent,1);
		%y = getWord(HudManager.currentHud.position,1) + %val;
		if(HudManager.constrain) %y = (%y >= %my) ? %my : ((%y < 0) ? 0 : %y);
		HudManager.currentHud.setPosition(getWord(HudManager.currentHud.position,0),%y);
	}
	else if(HudManager.hudAction $= "resize")
	{
		if(HudManager.currentDataBloc.noResize) return;
		%my = getWord(%parent.extent,1) - getWord(HudManager.currentHud.position,1);
		%y = getWord(HudManager.currentHud.extent,1) + %val;
		if(HudManager.constrain) %y = (%y >= %my) ? %my : ((%y < 0) ? 0 : %y);
		HudManager.currentHud.setExtent(getWord(HudManager.currentHud.extent,0),%y);
	}
	if((%mode != 0) && HudManager.loopMove) HudManager.loopID = schedule(200, 0, HM_hudY, %val, 2);
}

function HM_hudY_key(%val)
{
	HudManager.loopMove = true;
	HM_hudY(%val, 1);
}

//=============================================================================
//	So what do we do with these?!
//=============================================================================

function HudManager::displayMissionTypeGUI(%this)
{
	%this.forceToggleGUI(false);
	queryMasterGameTypes();
	%cnt = FilterEditMissionType.size();
	HM_MissionTypes.clear();
	if(!%this.typesInitialized)
	{
		%w = firstWord(HM_MissionTypes.extent);
  		HM_MissionTypes.addColumn(0,"MissionType",%w/3,%w/3,%w/3);
  		HM_MissionTypes.addColumn(1,"Layout",%w*2/3,%w*2/3,%w*2/3);
  		HM_MissionTypes.setSortColumn(0);
  		HM_MissionTypes.setSortIncreasing(true);
		%this.typesInitialized = true;
  	}
	for (%i=0; %i<%cnt; %i++)
	{
		%type = FilterEditMissionType.getTextByID(%i);
		if((%type !$= "Any") && (%type !$= "None"))
		{
			HM_gameTypes.add(%type, %idx++);
			if($HudManager::missionLayout[%type] !$= "") HM_MissionTypes.addRow(%cnt++, %type TAB  $HudManager::missionLayout[%type]);
			else HM_MissionTypes.addRow(%cnt++, %type);
		}
	}
}

function HudManager::changeName(%this, %oldName, %newName)
{
	if((%oldName $= "") || (%newName $= "")) return false;
	%oldBloc = %this.hudMap.value(%this.hudReverseMap.value(%oldName));
	if(%oldBloc $= "")
	{
		warn("HudManager::changeName -> cannot change name of \"" @ %name @ "\" - not a managed name");
		return false;
	}
	%this.addHUD(%newName, %oldObj);
	%this.removeHUD(%oldName, true);
	return true;
}

//=============================================================================
//	Packaged Calls
//=============================================================================

package HudManagerPkg
{
	function playGui::add(%this, %ctrl)
	{
		parent::add(%this,%ctrl);
		%bloc = HudManager.getHud(%ctrl);
		if(isObject(%bloc))
		{
			if(%bloc.position $= "") %bloc.position = %ctrl.position;
			if(%bloc.extent $= "") %bloc.extent = %ctrl.extent;
			if(%bloc.canvas !$= "" && isObject(%bloc.control))
			{
				if(%bloc.show == 0 && %bloc.canvas.isMember(%bloc.control)) %bloc.canvas.remove(%bloc.control);
				else if(%bloc.show == 1 && !%bloc.canvas.isMember(%bloc.control)) %bloc.canvas.add(%bloc.control);
			}
		}
	}

	function resizeChatHud(%val)
	{
		parent::resizeChatHud(%val);
		if(%val) HudManager.hudMap.value("ChatHud").read();
	}

	function ClientCmdSetHudMode(%mode, %type, %node)
	{
		HudManager.schedule(100,"setHuds");
		parent::ClientCmdSetHudMode(%mode, %type, %node);
		HudManager.schedule(500,"setHuds");
	}

	function ClientCmdDisplayHuds()
	{
		HudManager.schedule(100,"setHuds");
		parent::clientCmdDisplayHuds();
		HudManager.schedule(500,"setHuds");
	}

	function clientCmdSetWeaponsHudClearAll()
	{
		HudManager.schedule(100,"setHuds");
		parent::clientCmdSetWeaponsHudClearAll();
		HudManager.schedule(500,"setHuds");
	}

	function clientCmdTogglePlayHuds(%val)
	{
		HudManager.schedule(100,"setHuds");
		parent::clientcmdTogglePlayHuds(%val);
		HudManager.schedule(500,"setHuds");
	}

	function playGui::onWake(%this)
	{
		HudManager.schedule(100,"setHuds");
		parent::onWake(%this);
		if(!HudManager.SystemHudsInitialized) HudManager.schedule(750,"initSystemHuds");
		HudManager.schedule(500,"setHuds");
	}

	function EHReset()
	{
		HudManager.schedule(100,"setHuds");
		parent::EHReset();
		HudManager.schedule(500,"setHuds");
	}

	function escapeFromGame()
	{
		if(HudManager.displayed) HudManager.forceToggleGui(false);
		parent::escapeFromGame();
	}

	function resizeChatHud(%val)
	{
		parent::resizeChatHud(%val);
		HudManager.getHudByName("ChatHUD").read();
	}

	function dashboardHud::onResize(%this, %width, %height)
	{
		if(%this.position $= "0 0")
		{
			if(HudManager.getHud(dashboardHud).position $= "0 0")
			{
				parent::onResize(%this, %width, %height);
				HudManager.getHud(dashboardHud).read();
			}
			else HudManager.getHud(dashboardHud).write();
		}
	}

	function MainChatHud::nextChatHudLen(%this)
	{
		parent::nextChatHudLen(%this);
		HudManager.getHud(TR2EventPopup).write();
	}

	function createBonusPopup()
	{
		parent::createBonusPopup();
		HudManager.getHud(TR2EventPopup).write();
	}

	function TR2EventHud::resize(%this, %bigger)
	{
		%pos = TR2EventPopup.position;
		%ext = TR2EventPopup.extent;
		parent::resize(%this, %bigger);
		if($TR2Pref::EventHud::DockToChat)
		{
			if($TR2Pref::EventHud::ResizeUp)
			{
				TR2EventPopup.setPosition(
					firstWord(%pos),
					getWord(%pos,1) - (getWord(TR2EventPopup.extent,1) - getWord(%ext,1))
				);
				HudManager.getHud(TR2EventPopup).read();
			}
			else HudManager.getHud(TR2EventPopup).write();
		}
		else HudManager.getHud(TR2EventPopup).write();
	}

	function HudMover::addHud(%control, %name, %noResize)
	{
		HudManager.addHUD(%name, %control, %noResize);
		if(HudManager.hudMoverExists) parent::addHud(%control, %name);
	}

	function MessageHud::open(%this)
	{
		%offset = 8;
		if(%this.isVisible()) return;
		if(%this.isTeamMsg) %text = "TEAM:";
		else %text = "GLOBAL:";
		MessageHud_Text.setValue(%text);
		%textExtent = getWord(MessageHud_Text.extent, 0);
		%ctrlExtent = getWord(MessageHud_Frame.extent, 0);
		Canvas.pushDialog(%this);
		MessageHud_Edit.position = setWord(MessageHud_Edit.position, 0, %textExtent + %offset);
		MessageHud_Edit.extent = setWord(MessageHud_Edit.extent, 0, %ctrlExtent - %textExtent - (2 * %offset));
		%this.setVisible(true);
		deactivateKeyboard();
		MessageHud_Edit.makeFirstResponder(true);
	}

	function MessageHud::close(%this)
	{
		if(!%this.isVisible()) return;
		Canvas.popDialog(%this);
		%this.setVisible(false);
		if( $enableDirectInput ) activateKeyboard();
		MessageHud_Edit.setValue("");
	}

	function openVoteHud(%msgType, %msgString, %numClients, %passPercent)
	{
		alxPlay(VoteInitiatedSound, 0, 0, 0);
		voteHud.voting = true;
		voteHud.totalVotes   = 0;
		voteHud.size         = %numClients;
		voteHud.quorum       = (%numClients / 2);
		if(voteHud.quorum < 1) voteHud.quorum = 1;
		voteHud.pass = voteHud.quorum * %passPercent;
		voteHud.setPassValue(%passPercent);
		passHash.position = firstWord( mainVoteHud.extent) * %passPercent + 1 @ " -1";
		voteHud.setVisible(true);
		mainVoteHud.setVisible(true);
	}
};

if(!isObject(HM))
{
	if(isFile("hudmover/hudmover.cs")) HudManager.hudMoverExists = true;
	else
	{
		new ScriptObject(HM) {};
		new ScriptObject(hudmover) {};
		HM.bogus = hudmover.bogus = true;
	}
}
else
{
	if(!hudmover.bogus) HudManager.hudMoverExists = true;
	else HudManager.hudMoverExists = false;
}

if(isActivePackage(HudManagerPkg)) deactivatePackage(HudManagerPkg);
activatePackage(HudManagerPkg);

if(OuterChatHud.getGroup().getName() $= "") OuterChatHud.getGroup().setName("_OuterChat");
OuterChatHud.position = "0 0";

HudManager.readMapFile($HudManager::currentCfg);

Callback.add(AutoloadDone,"HudManager.changesDialog();");
Callback.add(onQuit,"export(\"$HudManager::*\", \"prefs/HudManager/HudManagerPrefs.cs\", false);");

Callback.add(onGameOver,"HudManager.forceToggleGui(false);");
Callback.add(onUserClientDrop,"HudManager.forceToggleGui(false);");
Callback.add(onMissionDropInfo,"HudManager.forceToggleGui(false);");

BindManager.addBind("HudManager","HM_guiToggle", true);
BindManager.addBind("HudManager Refresh","HM_refresh", true);
BindManager.addObsBind("HudManager","HM_guiToggle");
BindManager.addObsBind("HudManager Refresh","HM_refresh");

if(!isObject(HudManagerGui)) exec("scripts/UberGuy/HudManager.gui");
